package com.aopcloud.beam.app.api.coin.service.impl;

import com.aopcloud.beam.app.api.coin.entity.Coin;
import com.aopcloud.beam.app.api.coin.entity.CoinAction;
import com.aopcloud.beam.app.api.coin.mapper.CoinMapper;
import com.aopcloud.beam.app.api.coin.service.ICoinService;
import com.aopcloud.beam.app.core.Result;
import com.aopcloud.beam.app.core.ResultGenerator;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 货币类型表 服务实现类
 * </p>
 *
 * @author Enoch
 * @since 2021-02-24
 */
@Service
public class CoinServiceImpl extends ServiceImpl<CoinMapper, Coin> implements ICoinService {


    public Result add(CoinAction coinAction) {
        Coin coin = new Coin();
        BeanUtils.copyProperties(coinAction, coin);
        baseMapper.insert(coin);
        return ResultGenerator.genSuccessResult();
    }

    public Result edit(CoinAction coinAction) {
        Coin mCoin = baseMapper.selectOne(new QueryWrapper<Coin>().ge("id", coinAction.getId()));
        if (mCoin == null) {
            return ResultGenerator.genFailResult("货币类型不存在");
        }
        BeanUtils.copyProperties(coinAction, mCoin);
        baseMapper.update(mCoin, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult();
    }

    public Result delete(int deleteId) {
        baseMapper.deleteById(deleteId);
        return ResultGenerator.genSuccessResult("删除成功");
    }

    public Result getList(Integer pageIndex, Integer size) {
        Page<Coin> page = new Page<>(pageIndex, size);
        Page<Coin> userMyPage = page(page);
        return ResultGenerator.genSuccessResult(userMyPage.getRecords());
    }

    public Result create(CoinAction action) {
        Coin coin = new Coin();
        coin.setImgOssFile("" + action.getImgOssFile());
        coin.setCnName(action.getCnName());
        coin.setEnName(action.getEnName());
        coin.setCreateTime(LocalDateTime.now());
        coin.setUpdateTime(LocalDateTime.now());
        baseMapper.insert(coin);
        return ResultGenerator.genSuccessResult();
    }
}
