package com.aopcloud.beam.app.api.member.controller;


import com.alibaba.fastjson.JSON;
import com.aopcloud.beam.app.api.member.entity.Member;
import com.aopcloud.beam.app.api.member.entity.MemberActionBean;
import com.aopcloud.beam.app.api.member.service.impl.MemberServiceImpl;
import com.aopcloud.beam.app.api.user.entity.UpdateUserEntity;
import com.aopcloud.beam.app.api.user.entity.UserActionBean;
import com.aopcloud.beam.app.annotion.LoginRequired;
import com.aopcloud.beam.app.core.Result;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.Map;

/**
 * <p>
 * 会员表 前端控制器
 * </p>
 *
 * @author Enoch
 * @since 2021-02-24
 */
@RestController
@RequestMapping("/member")
public class MemberController {
    @Resource
    MemberServiceImpl beamUserService;


    @PostMapping("/register")
    public Result register(@RequestBody @Valid MemberActionBean register) {
        return beamUserService.register(register);
    }

    @PostMapping("/login")
    public Result login(@RequestBody MemberActionBean register) {
        return beamUserService.login(register);
    }

    @PostMapping("/logout")
    public Result logout(@RequestHeader @NotBlank String token) {
        return beamUserService.logout(token);
    }

    @LoginRequired
    @PostMapping("/edit")
    public Result edit(@RequestHeader @NotBlank String token, @RequestBody @Valid UpdateUserEntity user) {
        return beamUserService.edit(token, user);
    }

    @LoginRequired
    @GetMapping("/info")
    public Result getInfo(@RequestHeader @NotBlank String token) {
        return beamUserService.getInfo(token);
    }

    @PostMapping("/forgePassword")
    public Result getPassword(@RequestBody @Valid MemberActionBean actionBean) {
        return beamUserService.getPassword(actionBean);
    }

    @LoginRequired
    @PostMapping("/updatePassword")
    public Result updatePassword(@RequestBody Map<String, String> actionBean, @RequestHeader String token) {
        System.out.println(JSON.toJSONString(actionBean));
        return beamUserService.updatePassword(token, actionBean);
    }


    @GetMapping("/history")
    public Result history(@RequestHeader @NotBlank String token) {
        return beamUserService.history(token);
    }


}
