package com.aopcloud.beam.app.api.member.service.impl;

import com.alibaba.fastjson.JSON;
import com.aopcloud.beam.app.api.member.entity.Member;
import com.aopcloud.beam.app.api.member.entity.MemberActionBean;
import com.aopcloud.beam.app.api.member.mapper.MemberMapper;
import com.aopcloud.beam.app.api.member.service.IMemberService;
import com.aopcloud.beam.app.api.user.entity.BeamUser;
import com.aopcloud.beam.app.api.user.entity.HBean;
import com.aopcloud.beam.app.api.user.entity.UpdateUserEntity;
import com.aopcloud.beam.app.api.user.entity.UserActionBean;
import com.aopcloud.beam.app.common.Constants;
import  com.aopcloud.beam.app.core.AppException;;
import com.aopcloud.beam.app.core.Result;
import com.aopcloud.beam.app.core.ResultCode;
import com.aopcloud.beam.app.core.ResultGenerator;
import com.aopcloud.beam.app.util.IdUtil;
import com.aopcloud.beam.app.util.RegUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 会员表 服务实现类
 * </p>
 *
 * @author Enoch
 * @since 2021-02-24
 */
@Service
public class MemberServiceImpl extends ServiceImpl<MemberMapper, Member> implements IMemberService {
    public Result register(MemberActionBean userRegister) {
        if (!RegUtil.checkMobile(userRegister.getPhone())) {
            throw new AppException("手机号不正确");
        }
        System.out.println("phone:" + userRegister.getPhone());
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("phone", userRegister.getPhone()));
        if (user != null) {
            System.out.println("phone:" + user.getUsername() + "/" + user.getPhone());
            throw new AppException("手机号已经注册");
        }

        Member member = new Member();
        member.setMemberSn("" + IdUtil.create());
        member.setNickname(StringUtils.isEmpty(userRegister.getName()) ? IdUtil.genNickname() : userRegister.getName() + "");
        member.setUsername(StringUtils.isEmpty(userRegister.getName()) ? IdUtil.genNickname() : userRegister.getName() + "");

        if (StringUtils.isEmpty(userRegister.getOssFile())) {
            member.setAvatarOssFile(Constants.IMG_DEFAULT_USER_AVATAR);
        } else {
            member.setAvatarOssFile(userRegister.getOssFile());
        }
        member.setPhone(userRegister.getPhone());

        member.setPassword(userRegister.getPassword());
        member.setCreateTime(LocalDateTime.now());
        member.setUpdateTime(LocalDateTime.now());
        member.setSourceType(1);
        int id = baseMapper.insert(member);
        return ResultGenerator.genSuccessResult(id);
    }

    public Result login(MemberActionBean userRegister) {

        String token = null;//JwtUtil.createToken(user.getId() + "_" + user.getPhone());
        token = UUID.randomUUID().toString().replace("-", "");
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("phone", userRegister.getPhone()));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        if (!userRegister.getPassword().equals(user.getPassword())) {
            throw new AppException(-1, "密码不正确!", "密码不正确");
        }
        user.setToken(token);
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult(user);
    }

    public Result logout(String token) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("token", token));
        if (user != null) {
            user.setToken("");
            baseMapper.updateById(user);
        }
        return ResultGenerator.genSuccessResult("退出成功");
    }

    public Result edit(String token, UpdateUserEntity updateUser) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("token", token));
        if (user == null) {
            return ResultGenerator.genFailResult("用户不存在");
        }
        BeanUtils.copyProperties(updateUser, user);
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult();
    }

    public Result getInfo(String token) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("token", token));
        if (user == null) {
            throw new AppException("用户不存在");
        }
        return ResultGenerator.genSuccessResult(user);
    }

    public Result getPassword(MemberActionBean updateUser) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("phone", updateUser.getPhone()));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        user.setPassword(updateUser.getPassword());
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult(user);
    }

    public Result updatePassword(String token, Map<String,String> updateUser) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("token",token));
        if (user == null) {
            throw new AppException("用户不存在");
        }
        if (StringUtils.isEmpty(updateUser.get("oldPassword"))||StringUtils.isEmpty(updateUser.get("password"))) {
            throw new AppException("请输入旧密码或新密码");
        }
        if (!user.getPassword().equals(updateUser.get("oldPassword"))) {
            throw new AppException("旧密码不正确");
        }
        user.setPassword(updateUser.get("password"));
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult(user);
    }

    public Result getList(Integer pageIndex, Integer size) {
        System.out.println(pageIndex + "/" + size);
        Page<Member> page = new Page<>(pageIndex, size);
        Page<Member> userMyPage = baseMapper.selectPage(page, new QueryWrapper<>());
        System.out.println("" + JSON.toJSONString(userMyPage));
        Map map = new HashMap();
        map.put("totalCount", userMyPage.getTotal());
        map.put("totalPage", userMyPage.getPages());
        map.put("pageNo", userMyPage.getCurrent());
        map.put("pageSize", size);
        map.put("data", userMyPage.getRecords());
        return ResultGenerator.genSuccessResult(map);
    }


    public Result updateStatus(Map<String, Object> params) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("id", params.get("id")));
        if (user == null) {
            throw new AppException("用户不存在");
        }
        user.setStatus((Integer) params.get("status"));
        baseMapper.updateById(user);
        return ResultGenerator.genSuccessResult();
    }


    public Result history(String token) {
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("token", token));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        String url = "http://oss.mxzhe.cn/img/img_coin_1612004588012.png";
        List<HBean> list = new ArrayList<>();
        //String coinName, String coinN, String coinIcon, String type, String createTime, String orderNo, String id, int sum, String tag) {
        HBean hBean = new HBean("比特币", "BTC", url, 1, "2021-02-13 19:00", "8qwoiqwndonqwkdqkd", "123", 8980670.01f, "区块奖励");
        list.add(hBean);
        hBean = new HBean("比特现金", "BCH", url, 2, "2021-02-14 19:00", "8qwoiqwndonqwkdqkd", "1212e1", 24520.01f, "区块奖励");
        list.add(hBean);
        hBean = new HBean("莱特币", "LTC", url, 1, "2021-02-13 19:00", "8qwoiqwndonqwkdqkd", "asdasd123123asdasda", 892340.01f, "区块奖励");
        list.add(hBean);
        return ResultGenerator.genSuccessResult(list);
    }


    public Member getMemberByToken(String token) {
        System.out.println("getMemberByToken:"+token);
        if (StringUtils.isEmpty(token)){
            throw new AppException(ResultCode.LOGIN_INVALID.code(),"登录已过期");
        }
        Member user = baseMapper.selectOne(new QueryWrapper<Member>().eq("token", token));
        if (user == null) {
            throw new AppException(ResultCode.LOGIN_INVALID.code(),"登录已过期");
        }
        return user;
    }
}
