package com.aopcloud.beam.app.api.order.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aopcloud.beam.app.api.app.entity.HomeNotice;
import com.aopcloud.beam.app.api.coin.entity.Coin;
import com.aopcloud.beam.app.api.coin.entity.CoinPackage;
import com.aopcloud.beam.app.api.coin.service.impl.CoinPackageServiceImpl;
import com.aopcloud.beam.app.api.coin.service.impl.CoinServiceImpl;
import com.aopcloud.beam.app.api.member.entity.Member;
import com.aopcloud.beam.app.api.member.entity.MemberCoinAddress;
import com.aopcloud.beam.app.api.member.service.impl.MemberCoinAddressServiceImpl;
import com.aopcloud.beam.app.api.member.service.impl.MemberCoinRecordServiceImpl;
import com.aopcloud.beam.app.api.member.service.impl.MemberServiceImpl;
import com.aopcloud.beam.app.api.order.entity.*;
import com.aopcloud.beam.app.api.order.mapper.OrderMapper;
import com.aopcloud.beam.app.api.order.service.IOrderService;
import com.aopcloud.beam.app.common.PayConfig;
import com.aopcloud.beam.app.dotwallet.DotWalletManager;
import com.aopcloud.beam.app.util.OrderIdUtil;
import com.aopcloud.beam.app.core.AppException;;
import com.aopcloud.beam.app.core.Result;
import com.aopcloud.beam.app.core.ResultGenerator;
import com.aopcloud.beam.app.http.OkHttpUtils;
import com.aopcloud.beam.app.util.MD5Util;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import okhttp3.*;
import org.apache.commons.lang.math.RandomUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;

/**
 * <p>
 * 订单表 服务实现类
 * </p>
 *
 * @author Enoch
 * @since 2021-02-24
 */
@Service
public class OrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements IOrderService {


    @Resource
    public MemberServiceImpl memberService;
    @Resource
    public CoinServiceImpl coinService;
    @Resource
    public MemberCoinAddressServiceImpl coinAddressService;
    @Resource
    public CoinPackageServiceImpl coinPackageService;
    @Resource
    public OrderPayRecordsServiceImpl payRecordsService;
    @Resource
    public MemberCoinRecordServiceImpl coinRecordService;


    public Result create(String token, OrderCreateEntity createEntity) {
//        System.out.println("create order:"+JSON.toJSONString(createEntity));
        Member member = memberService.getMemberByToken(token);
        Order order = new Order();
        order.setOrderSn(OrderIdUtil.createOrderSn(member.getMemberSn()));
        order.setMemberId(member.getId());
        order.setMemberSn(member.getMemberSn());
        order.setSourceType(2);
        order.setStatus(0);
        order.setOrderType(0);


        Coin coin = coinService.getById(createEntity.coinId);
        if (coin == null) {
            throw new AppException("货币不存在");
        }

        CoinPackage coinPackage = coinPackageService.getById(createEntity.skuId);
        if (coinPackage == null) {
            throw new AppException("算力包不存在");
        }

        QueryWrapper<Order> condition = new QueryWrapper<Order>();
        condition.orderByDesc("lease_end_time")
                .last("limit 1");
        Order  lastOrder= baseMapper.selectOne(condition);

        if (lastOrder != null&&lastOrder.getLeaseEndTime().toEpochSecond(ZoneOffset.of("+8"))>createEntity.startTime) {
            throw new AppException("开始时间应大于："+lastOrder.getLeaseEndTime());
        }

        MemberCoinAddress memberCoinAddress = coinAddressService.getOne(
                new QueryWrapper<MemberCoinAddress>()
                        .eq("address", createEntity.coinAddress)
                        .eq("coin_id", createEntity.coinId)
                        .eq("member_id", member.getId())
        );
        if (memberCoinAddress == null) {
            memberCoinAddress = new MemberCoinAddress();
            memberCoinAddress.setAddress(createEntity.coinAddress);
            memberCoinAddress.setCoinId(createEntity.coinId);
            memberCoinAddress.setMemberId(member.getId());
            memberCoinAddress.setDescription("");
            coinAddressService.save(memberCoinAddress);
        }

        order.setCoinId(createEntity.coinId);
        order.setCoinAddress(memberCoinAddress.getAddress());
        order.setCoinAddressId(memberCoinAddress.getId());
        order.setOssImg(coin.getImgOssFile());

        LocalDateTime leaseStartTime = LocalDateTime.ofEpochSecond(createEntity.startTime, 0, ZoneOffset.ofHours(8));
        LocalDateTime leaseEndTime = LocalDateTime.ofEpochSecond(createEntity.endTime, 0, ZoneOffset.ofHours(8));

        order.setSukId(createEntity.skuId);
        order.setLeaseStartTime(leaseStartTime);
        order.setLeaseEndTime(leaseEndTime);
        order.setDuration(createEntity.endTime - createEntity.startTime);

        long s = createEntity.endTime - createEntity.startTime;
        long price = s * coinPackage.getPrice().longValue();

        order.setTotalAmount(BigDecimal.valueOf(price));
        order.setPayAmount(BigDecimal.valueOf(price));
        order.setPayAmount(BigDecimal.valueOf(0.01));
        order.setDiscountAmount(BigDecimal.valueOf(0));

        save(order);
        return ResultGenerator.genSuccessResult(order);
    }


    String payInfo;

    public Result pay(String token, String orderSn, String payChannel) {
        Member member = memberService.getMemberByToken(token);

        Order order = getOne(new QueryWrapper<Order>().eq("order_sn", orderSn));
        if (order == null) {
            throw new AppException("订单不存在");
        }

        if (order.getStatus() != 0) {
            throw new AppException("订单已经支付");
        }


        long timestamp = System.currentTimeMillis() / 1000;
        String appId = PayConfig.PAY_APP_ID;
        String payKey = PayConfig.PAY_KEY;
        String subject = "算力租赁";
        String price = "" + order.getPayAmount();
        String notifyUrl = PayConfig.PAY_NOTIFY_URL;
        String extendsParam = "算力租赁";
        Map<String, String> map = new HashMap<>();
        map.put("appid", appId);
        map.put("timestamp", "" + timestamp);
        map.put("subject", subject);
        map.put("orderid", orderSn);
        map.put("price", price);
        map.put("notifyurl", notifyUrl);
        map.put("channel", "" + payChannel);
        map.put("extends_param", extendsParam);

        String s = appId + timestamp + orderSn + payChannel + subject + price + notifyUrl + extendsParam + payKey;
        System.out.println("计算签名值 s：" + s);
        String sign = MD5Util.encrypt(s);
        System.out.println("sign：" + MD5Util.encrypt(s));
        map.put("sign", sign);
        map.put("user_id", member.getMemberSn());
        map.put("front_url", "");
        System.out.println("JSON：" + JSON.toJSONString(map));

        String query = urlEncodeUTF8(map);
        String url = PayConfig.PAY_ORDER_CREATE_URL + "?" + query;
        System.out.println("url：" + url);
        Response response = null;
        try {
            response = OkHttpUtils
                    .post()
                    .url(PayConfig.PAY_ORDER_CREATE_URL)
                    .params(map)
                    .build().execute();
            String p = response.body().string();
            System.out.println("-------:" + p);
            JSONObject jsonObject = JSON.parseObject(p);
            if (jsonObject.get("code").equals("error")) {
                throw new AppException(jsonObject.get("msg").toString());
            }
            JSONObject data = JSON.parseObject(jsonObject.get("data").toString());


            OrderPayRecords payRecords = new OrderPayRecords();
            payRecords.setPrepayid(data.get("id").toString());
            payRecords.setOrderId(order.getId());
            payRecords.setOrderSn(order.getOrderSn());
            payRecords.setPaymentChannel(Integer.valueOf(payChannel));
            payRecords.setPaymentSubject(subject);
            payRecords.setPaymentAmount(order.getPayAmount());
            payRecords.setExtendsParam(extendsParam);
            payRecordsService.save(payRecords);
            System.out.println("payInfo：" + jsonObject.get("data"));
            return ResultGenerator.genSuccessResult(jsonObject.get("data"));
        } catch (IOException e) {
            e.printStackTrace();
            throw new AppException("获取支付内容失败");
        }
    }


    public Result queryOrderPayStatus(String orderSn) {

        Order order = getOne(new QueryWrapper<Order>().eq("order_sn", orderSn));
        OrderPayRecords payRecords = payRecordsService.getOne(new QueryWrapper<OrderPayRecords>()
                .eq("order_sn", orderSn).orderByDesc(true, "create_time").last("limit 1"));
        if (order == null) {
            throw new AppException("订单不存在");
        }
        long timestamp = System.currentTimeMillis() / 1000;
        String appId = PayConfig.PAY_APP_ID;
        String payKey = PayConfig.PAY_KEY;
        String id = payRecords.getPrepayid();
        Map<String, String> map = new HashMap<>();
        map.put("appid", appId);
        map.put("timestamp", "" + timestamp);
        map.put("id", id);
        String s = appId + timestamp + id + payKey;
        System.out.println("计算签名值 s：" + s);
        String sign = MD5Util.encrypt(s);
        map.put("sign", sign);

        Response response = null;
        try {
            response = OkHttpUtils
                    .post()
                    .url(PayConfig.PAY_ORDER_QUERY_URL)
                    .params(map)
                    .build().execute();
            payInfo = response.body().string();

            System.out.println("-------:" + payInfo);
            JSONObject jsonObject = JSON.parseObject(payInfo);
            if (jsonObject.get("code").equals("error")) {
                throw new AppException(jsonObject.get("msg").toString());
            }
            JSONObject data = JSON.parseObject(jsonObject.get("data").toString());

            System.out.println("payInfo：" + payInfo);
            return ResultGenerator.genSuccessResult(data);
        } catch (IOException e) {
            e.printStackTrace();
            throw new AppException("获取支付内容失败");
        }
    }


    static String urlEncodeUTF8(String s) {
        try {
            return URLEncoder.encode(s, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            throw new UnsupportedOperationException(e);
        }
    }

    static String urlEncodeUTF8(Map<?, ?> map) {
        StringBuilder sb = new StringBuilder();
        for (Map.Entry<?, ?> entry : map.entrySet()) {
            if (sb.length() > 0) {
                sb.append("&");
            }
            sb.append(String.format("%s=%s",
                    urlEncodeUTF8(entry.getKey().toString()),
                    urlEncodeUTF8(entry.getValue().toString())
            ));
        }
        return sb.toString();
    }

    public Result list(String token) {
        return ResultGenerator.genSuccessResult(list());
    }

    public Result getPayState(String orderSn) {
        Order order = getOne(new QueryWrapper<Order>().eq("order_sn", orderSn));
        if (order == null) {
            throw new AppException("订单不存在");
        }
        return queryOrderPayStatus(orderSn);
    }

    public Result getOrderDetail(String orderSn) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.eq("order_sn", orderSn);
        Order order = getOne(new QueryWrapper<Order>()
                .eq("order_sn", orderSn)
        );
        if (order == null) {
            throw new AppException("订单不存在");
        }
        List hash = new ArrayList<>();
        for (int i = 0; i < 3; i++) {
            Map map = new HashMap();
            map.put("time", "12:" + (i + 1) + "0");
            map.put("hash", RandomUtils.nextInt(3) * 100);
            hash.add(map);
        }
        Coin coin = coinService.getById(order.getCoinId());
        CoinPackage coinPackage = coinPackageService.getById(order.getSukId());

        long startTime = order.getLeaseStartTime().toEpochSecond(ZoneOffset.of("+8"));
        long endTime = order.getLeaseStartTime().toEpochSecond(ZoneOffset.of("+8"));
        List<BlockBean> blockList = DotWalletManager.getInstance().getBlockList(startTime, endTime);

        if (blockList.isEmpty() || blockList.size() == 0) {
            BlockBean blockBean = new BlockBean();
            blockBean.setPuid(0);
            blockBean.setWorkerFullName("123");
            blockBean.setHeight(RandomUtils.nextInt());
            blockBean.setHash(""+RandomUtils.nextInt());
            blockBean.setRewards(RandomUtils.nextInt());
            blockBean.setCreatedAt(System.currentTimeMillis() / 1000);
            blockList.add(blockBean);
        }
        System.out.println("coin:" + JSON.toJSONString(coin));
        OrderDetailBean orderDetailBean = new OrderDetailBean();
        BeanUtils.copyProperties(order, orderDetailBean);
        BeanUtils.copyProperties(coin, orderDetailBean);
        orderDetailBean.setHash(hash);
        orderDetailBean.setBlockList(blockList);
        orderDetailBean.setCoinPackage(coinPackage);


        int rewards = 0;
        for (int i = 0; i < blockList.size(); i++) {
            rewards = rewards + blockList.get(i).getRewards();
        }
        orderDetailBean.setTotalProfit(rewards);

        return ResultGenerator.genSuccessResult(orderDetailBean);
    }

    public Result getBasicInfo() {
        QueryWrapper<Order> condition = new QueryWrapper<Order>();
        condition.orderByDesc("lease_end_time")
                .last("limit 1");
        Order order = baseMapper.selectOne(condition);

        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        Map map = new HashMap();
        if (order!=null){
            map.put("startTime",order.getLeaseEndTime());
        }else {
            map.put("startTime",dateFormat.format(new Date()));
        }
        map.put("coinList",coinService.list());
        map.put("skuList",coinPackageService.list());
        return ResultGenerator.genSuccessResult(map);
    }
}
