package com.aopcloud.beam.app.api.user.service.impl;

import com.alibaba.fastjson.JSON;
import com.aopcloud.beam.app.api.user.entity.BeamUser;
import com.aopcloud.beam.app.api.user.entity.HBean;
import com.aopcloud.beam.app.api.user.entity.UpdateUserEntity;
import com.aopcloud.beam.app.api.user.entity.UserActionBean;
import com.aopcloud.beam.app.api.user.mapper.BeamUserMapper;
import com.aopcloud.beam.app.api.user.service.IBeamUserService;
import com.aopcloud.beam.app.common.Constants;
import com.aopcloud.beam.app.util.RSAUtils;
import  com.aopcloud.beam.app.core.AppException;;
import com.aopcloud.beam.app.core.Result;
import com.aopcloud.beam.app.core.ResultGenerator;
import com.aopcloud.beam.app.util.IdUtil;
import com.aopcloud.beam.app.util.RegUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author Enoch
 * @since 2021-01-28
 */
@Service
public class BeamUserServiceImpl extends ServiceImpl<BeamUserMapper, BeamUser> implements IBeamUserService {


    public Result register(UserActionBean userRegister) {
        if (!RegUtil.checkMobile(userRegister.getPhone())) {
            throw new AppException(-1, "手机号不正确!", "手机号不正确");
        }
        List user = baseMapper.selectList(new QueryWrapper<BeamUser>().ge("phone", userRegister.getPhone()));
        if (!user.isEmpty() && user.size() > 0) {
            throw new AppException(1001, "用户已存在!", "用户已存在");
        }

        BeamUser beamUser = new BeamUser();
        beamUser.setAccount("" + IdUtil.create());
        beamUser.setName("会员" + IdUtil.create() + "");
        beamUser.setAvatar(Constants.IMG_DEFAULT_USER_AVATAR);
        beamUser.setPhone(userRegister.getPhone());
        beamUser.setSalt(userRegister.getPhone());
        beamUser.setPassword(userRegister.getPassword());
        beamUser.setCreatedTime(LocalDateTime.now());
        beamUser.setCreatedBy(userRegister.getPhone());
        beamUser.setUpdateTime(LocalDateTime.now());
        beamUser.setUpdateBy(userRegister.getPhone());
        int id = baseMapper.insert(beamUser);
        return ResultGenerator.genSuccessResult(id);
    }


    public Result login(UserActionBean userRegister) {

        String token = null;//JwtUtil.createToken(user.getId() + "_" + user.getPhone());
        token = UUID.randomUUID().toString().replace("-", "");
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("phone", userRegister.getPhone()));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        if (!userRegister.getPassword().equals("sunaliyun_" + userRegister.getPassword())) {
        }
//                throw new AppException(-1, "密码不正确!", "密码不正确");
//            }
        String password = null;


//            password = new String(RSAUtils.decryptByPrivateKey(userRegister.getPassword().getBytes(StandardCharsets.UTF_8),RSAUtils.PRIVATE_KEY.getBytes(StandardCharsets.UTF_8)));
//            System.out.println("password:"+password);
//            if (!password.equals("sunaliyun_"+userRegister.getPassword())) {
//                throw new AppException(-1, "密码不正确!", "密码不正确");
//            }
        user.setToken(token);
        System.out.println("-------:" + JSON.toJSONString(user));
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult(user);
    }

    public Result logout(String token) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("token", token));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        user.setToken("");
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult("退出成功");
    }

    public Result edit(String token, UpdateUserEntity updateUser) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("token", token));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        BeanUtils.copyProperties(updateUser, user);
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult();
    }

    public Result getInfo(String token) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("token", token));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        return ResultGenerator.genSuccessResult(user);
    }

    public Result getPassword(UserActionBean updateUser) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("phone", updateUser.getPhone()));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        user.setPassword(updateUser.getPassword());
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult(user);
    }

    public Result updatePassword(UserActionBean updateUser) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("phone", updateUser.getPhone()));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        user.setPassword(updateUser.getPassword());
        baseMapper.update(user, new UpdateWrapper<>());
        return ResultGenerator.genSuccessResult("修改成功");
    }

    public Result history(String token) {
        BeamUser user = baseMapper.selectOne(new QueryWrapper<BeamUser>().ge("token", token));
        if (user == null) {
            throw new AppException(-1, "用户不存在!", "用户不存在");
        }
        String url = "http://oss.mxzhe.cn/img/img_coin_1612004588012.png";
        List<HBean> list = new ArrayList<>();
        //String coinName, String coinN, String coinIcon, String type, String createTime, String orderNo, String id, int sum, String tag) {
        HBean hBean = new HBean("比特币", "BTC", url, 1, "2021-02-13 19:00", "8qwoiqwndonqwkdqkd", "123", 8980670.01f, "区块奖励");
        list.add(hBean);
        hBean = new HBean("比特现金", "BCH", url, 2, "2021-02-14 19:00", "8qwoiqwndonqwkdqkd", "1212e1", 24520.01f, "区块奖励");
        list.add(hBean);
        hBean = new HBean("莱特币", "LTC", url, 1, "2021-02-13 19:00", "8qwoiqwndonqwkdqkd", "asdasd123123asdasda", 892340.01f, "区块奖励");
        list.add(hBean);
        return ResultGenerator.genSuccessResult(list);
    }
}
