package com.aopcloud.beam.app.configurer;

import com.alibaba.fastjson.JSON;
import  com.aopcloud.beam.app.core.AppException;;
import com.aopcloud.beam.app.core.Result;
import com.aopcloud.beam.app.core.ResultCode;
import com.aopcloud.beam.app.core.ResultGenerator;
import org.springframework.http.HttpStatus;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.NoHandlerFoundException;

import java.util.List;


/**
 * 全局异常
 */
@ControllerAdvice
public class GlobalExceptionHandler {
//    @InitBinder
//    public void initBinder(WebDataBinder binder) {
//        // 方法1，注册converter
//        GenericConversionService genericConversionService = (GenericConversionService) binder.getConversionService();
//        if (genericConversionService != null) {
//            genericConversionService.addConverter(new DateConverter());
//        }
//
//        // 方法2，定义单格式的日期转换，可以通过替换格式，定义多个dateEditor，代码不够简洁
//        DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        CustomDateEditor dateEditor = new CustomDateEditor(df, true);
//        binder.registerCustomEditor(Date.class, dateEditor);
//
//
//        // 方法3，同样注册converter
//        binder.registerCustomEditor(Date.class, new PropertyEditorSupport() {
//            @Override
//            public void setAsText(String text) throws IllegalArgumentException {
//                setValue(new DateConverter().convert(text));
//            }
//        });
//
//    }

    @ResponseBody
    @ExceptionHandler(value = NoHandlerFoundException.class)
    @ResponseStatus(HttpStatus.NOT_FOUND)
    public Result exceptionHandler(NoHandlerFoundException e) {
        e.printStackTrace();
        return ResultGenerator.genFailResult(ResultCode.NOT_FOUND,e.getRequestURL()+"接口不存在");
    }


    @ExceptionHandler
    @ResponseBody
    @ResponseStatus(code=HttpStatus.BAD_REQUEST)
    public Result exceptionHandler(MethodArgumentNotValidException e) {
        List<FieldError> bindingResult = e.getBindingResult().getFieldErrors();
        System.out.println(JSON.toJSONString(bindingResult.get(0)));
        return ResultGenerator.genFailResult(bindingResult.get(0).getDefaultMessage());
    }
    @ExceptionHandler
    @ResponseBody
    @ResponseStatus(code=HttpStatus.BAD_REQUEST)
    public Result exceptionHandler(AppException e) {
        return ResultGenerator.genFailResult(e.getCode(),e.getMessage());
    }
}
