package com.aopcloud.beam.app.delay;

import com.aopcloud.beam.app.api.order.entity.Order;
import com.aopcloud.beam.app.api.order.service.impl.OrderServiceImpl;
import com.aopcloud.beam.app.configurer.AppContextHolder;
import com.aopcloud.beam.app.dotwallet.DotWalletManager;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.DelayQueue;
import java.util.concurrent.Executors;

@Component
public class DelayQueueManager implements CommandLineRunner, InitializingBean {
    private final Logger logger = LoggerFactory.getLogger(DelayQueueManager.class);
    private DelayQueue<DelayTask> delayQueue = new DelayQueue<>();

    /**
     * 加入到延时队列中
     *
     * @param task
     */
    public void put(DelayTask task) {
        logger.info("加入延时任务：{}", task);
        delayQueue.put(task);
        logger.info("延时任务SIZE：{}", delayQueue.size());
    }


    /**
     * 取消延时任务
     *
     * @param task
     * @return
     */
    public boolean remove(DelayTask task) {
        logger.info("取消延时任务：{}", task);
        return delayQueue.remove(task);
    }

    /**
     * 取消延时任务
     *
     * @param taskId
     * @return
     */
    public boolean remove(int taskId) {
        return remove(new DelayTask((taskId), "", 0));
    }


    @Override
    public void run(String... args) {
        logger.info("初始化延时队列");
        Executors.newSingleThreadExecutor().execute((this::executeThread));
    }

    /**
     * 延时任务执行线程
     */
    private void executeThread() {
        while (true) {
            try {
                DelayTask task = delayQueue.take();
                processTask(task);
            } catch (InterruptedException e) {
                e.printStackTrace();
                logger.error("执行延时任务ERROR：{}", e.getMessage());
            }
        }
    }

    /**
     * 内部执行延时任务
     *
     * @param task
     */
    private void processTask(DelayTask task) {
        logger.info("执行延时任务：{}", task);
        //根据task中的data自定义数据来处理相关逻辑，例 if (task.getData() instanceof XXX) {}
        orderService = AppContextHolder.getBean(OrderServiceImpl.class);
        Order order = (Order) task.getData();
        if (order.getLeaseStatus().equals("0")) {
            boolean s = DotWalletManager.getInstance().startSolo(order.getCoinAddress());
            order.setLeaseStatus(s ? "1" : "0");
        } else if (order.getLeaseStatus().equals("1")) {
            boolean s = DotWalletManager.getInstance().stopSolo(order.getCoinAddress());
            order.setLeaseStatus(s ? "2" : "1");
        }
        orderService.saveOrUpdate(order);
        System.out.println("task order:" + orderService.getById(order.getId()));
    }


    private OrderServiceImpl orderService;
    private static DelayQueueManager queueManager;

    public static DelayQueueManager getQueueManager() {
        return queueManager;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        queueManager = this;
    }
}