package com.aopcloud.beam.app.delay;


import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.Delayed;
import java.util.concurrent.TimeUnit;

public class DelayTask<T> implements Delayed {
    final private T data;
    final private long expire;
    private int identifier;

    /**
     * 构造延时任务
     *
     * @param id     id
     * @param data   业务数据
     * @param expire 任务延时时间（ms）
     */
    public DelayTask(int id, T data, long expire) {
        super();
        this.identifier = id;
        this.data = data;
        this.expire = expire + System.currentTimeMillis();

    }

    public T getData() {
        return data;
    }

    public long getExpire() {
        return expire;
    }

    public int getId() {
        return identifier;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof DelayTask) {
            return this.getId()==getId();
        }
        return false;
    }

    @Override
    public String toString() {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return "{" + "data:" + data.toString() + "," + "expire:" + dateFormat.format(new Date(expire)) + "}";
    }

    @Override
    public long getDelay(TimeUnit unit) {
        //        //下面用到unit.convert()方法，其实在这个小场景不需要用到，只是学习一下如何使用罢了
        long e= this.expire-System.currentTimeMillis();
        System.out.println("delay:"+e);
        return unit.convert(e, TimeUnit.MILLISECONDS);
    }

    @Override
    public int compareTo(Delayed o) {
        //        //这里根据取消时间来比较，如果取消时间小的，就会优先被队列提取出来
        long delta = getDelay(TimeUnit.MILLISECONDS) - o.getDelay(TimeUnit.MILLISECONDS);
        return (int) delta;
    }
}