package com.aopcloud.beam.app.scheduled;


import com.alibaba.fastjson.JSON;
import com.aopcloud.beam.app.api.app.entity.HomeBean;
import com.aopcloud.beam.app.api.app.entity.HomeCoin;
import com.aopcloud.beam.app.api.app.entity.HomeNotice;
import com.aopcloud.beam.app.api.app.service.impl.HomeCoinServiceImpl;
import com.aopcloud.beam.app.api.app.service.impl.HomeNoticeServiceImpl;
import com.aopcloud.beam.app.api.coin.entity.Coin;
import com.aopcloud.beam.app.api.coin.service.impl.CoinServiceImpl;
import com.aopcloud.beam.app.api.member.entity.Member;
import com.aopcloud.beam.app.configurer.AppContextHolder;
import com.aopcloud.beam.app.scheduled.entity.BCHMetadata;
import com.aopcloud.beam.app.scheduled.entity.BSVMetadata;
import com.aopcloud.beam.app.scheduled.entity.BTCMetadata;
import com.aopcloud.beam.app.http.OkHttpUtils;
import com.aopcloud.beam.app.http.callback.StringCallback;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.generator.config.IFileCreate;
import okhttp3.Call;
import org.apache.commons.lang.StringUtils;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
public class CoinMetadataScheduled {

    @Resource
    private CoinServiceImpl coinService;

    @Resource
    private HomeNoticeServiceImpl homeNoticeService;

//    @Scheduled(fixedDelay = 1 * 60 * 1000)
    private void configureTasks() {
        if (coinService == null) {
            coinService = AppContextHolder.getBean(CoinServiceImpl.class);
        }
        if (homeNoticeService == null) {
            homeNoticeService = AppContextHolder.getBean(HomeNoticeServiceImpl.class);
        }

        System.err.println("执行静态定时任务 start 时间: " + LocalDateTime.now());
        getBTCMetadata();
        getBCHMetadata();
        getBSVMetadata();
        System.err.println("执行静态定时任务 end 时间: " + LocalDateTime.now());
    }

    String bsv = "https://bsv.tokenview.com/api/blocks/bchsv/1/1";
    String bch = "https://bch.tokenview.com/api/blocks/bch/1/1";
    String btc = "https://chain.api.btc.com/v3/block/latest";

    private void getBTCMetadata() {
        OkHttpUtils.get().url(btc).build().execute(new StringCallback() {
            @Override
            public void onError(Call call, Exception e, int id) {

            }

            @Override
            public void onResponse(String response, int id) {
                BTCMetadata btcMetadata = JSON.parseObject(response, BTCMetadata.class);
                if (btcMetadata.getStatus().equals("success")) {

                    int no = btcMetadata.getData().getHeight();

                    QueryWrapper<HomeNotice> condition = new QueryWrapper<>();
                    condition.eq("notice_sn", "" + no)
                            .last("limit 1");
                    HomeNotice notice = homeNoticeService.getBaseMapper().selectOne(new QueryWrapper<HomeNotice>()
                            .eq("notice_sn", "" + no).last("limit 1"));
                    System.out.println("sn:" + no);
                    if (notice == null) {
                        notice = new HomeNotice();
                        notice.setCoinId(96);
                        notice.setNoticeSn("" + no);
                        notice.setCoinName("BTC");
                    }

                    notice.setRemarks(JSON.toJSONString(btcMetadata.getData()));
                    notice.setNoticeSource(btcMetadata.getData().getExtras().getPool_name());
                    notice.setNoticeTimestamp(btcMetadata.getData().getTimestamp());
                    BigDecimal decimal = new BigDecimal(0.00000001);
                    BigDecimal r = btcMetadata.getData().getReward_block().add(btcMetadata.getData().getReward_fees()).multiply(decimal);
                    notice.setProfit(r);
                    boolean save = homeNoticeService.saveOrUpdate(notice);
                    System.out.println("btc save:" + save);
                    QueryWrapper<HomeNotice> queryWrapper =
                            new QueryWrapper<HomeNotice>();
                    queryWrapper
                            .orderByDesc("notice_timestamp")
                            .eq("coin_id", 96)
                            .last("limit 1");
                    List<HomeNotice> page = homeNoticeService.getBaseMapper().selectList(queryWrapper);
                    System.out.println("btc getNoticeTimestamp:" + page.get(0).getNoticeTimestamp());
                    Coin coin = coinService.getById(96);
                    coin.setReward(r);
                    coin.setLatestTime(page.get(0).getNoticeTimestamp());
                    boolean update = coinService.updateById(coin);
                    System.out.println("btc update:" + update);


                }
            }
        });
    }

    private void getBCHMetadata() {
        OkHttpUtils.get().url(bch).build().execute(new StringCallback() {
            @Override
            public void onError(Call call, Exception e, int id) {

            }

            @Override
            public void onResponse(String response, int id) {
                BCHMetadata bchMetadata = JSON.parseObject(response, BCHMetadata.class);
                if (bchMetadata.getCode() == 1) {

                    String no = "" + bchMetadata.getData().get(0).getBlock_no();


                    QueryWrapper<HomeNotice> condition = new QueryWrapper<>();
                    condition.eq("notice_sn", "" + no)
                            .last("limit 1");
                    HomeNotice notice = homeNoticeService.getBaseMapper().selectOne(new QueryWrapper<HomeNotice>()
                            .eq("notice_sn", "" + no).last("limit 1"));
                    System.out.println("sn:" + no);
                    if (notice == null) {
                        notice = new HomeNotice();
                        notice.setCoinId(97);
                        notice.setNoticeSn(no);
                        notice.setCoinName("BCH");
                    }

                    notice.setRemarks(JSON.toJSONString(bchMetadata.getData()));
                    notice.setNoticeSource(!StringUtils.isEmpty(bchMetadata.getData().get(0).getMiner())
                            ? bchMetadata.getData().get(0).getMiner() : "" + bchMetadata.getData().get(0).getMinerAlias());
                    notice.setNoticeTimestamp(bchMetadata.getData().get(0).getTime());
                    BigDecimal reward = new BigDecimal(bchMetadata.getData().get(0).getReward());
                    notice.setProfit(reward);
                    boolean save = homeNoticeService.saveOrUpdate(notice);
                    QueryWrapper<HomeNotice> queryWrapper =
                            new QueryWrapper<HomeNotice>();
                    queryWrapper
                            .orderByDesc("notice_timestamp")
                            .eq("coin_id", 97)
                            .last("limit 1");
                    List<HomeNotice> page = homeNoticeService.getBaseMapper().selectList(queryWrapper);
                    System.out.println("BCH getNoticeTimestamp:" + page.get(0).getNoticeTimestamp());
                    Coin coin = coinService.getById(97);
                    coin.setReward(reward);
                    coin.setLatestTime(page.get(0).getNoticeTimestamp());
                    boolean update = coinService.updateById(coin);
                    System.out.println("BCH update:" + update);
                }

            }
        });

    }

    private void getBSVMetadata() {
        OkHttpUtils.get().url(bsv).build().execute(new StringCallback() {
            @Override
            public void onError(Call call, Exception e, int id) {

            }

            @Override
            public void onResponse(String response, int id) {
                BSVMetadata bsvMetadata = JSON.parseObject(response, BSVMetadata.class);

                if (bsvMetadata.getCode() == 1) {
                    String no = "" + bsvMetadata.getData().get(0).getBlockNo();


                    QueryWrapper<HomeNotice> condition = new QueryWrapper<>();
                    condition.eq("notice_sn", "" + no)
                            .last("limit 1");
                    HomeNotice notice = homeNoticeService.getBaseMapper().selectOne(new QueryWrapper<HomeNotice>()
                            .eq("notice_sn", "" + no).last("limit 1"));
                    System.out.println("sn:" + no);
                    if (notice == null) {
                        notice = new HomeNotice();
                        notice.setCoinId(98);
                        notice.setNoticeSn(no);
                        notice.setCoinName("BSV");
                    }
                    notice.setRemarks(JSON.toJSONString(bsvMetadata.getData()));
                    notice.setNoticeSource(bsvMetadata.getData().get(0).getMiner());
                    notice.setNoticeTimestamp(bsvMetadata.getData().get(0).getTime());
                    BigDecimal reward = new BigDecimal(bsvMetadata.getData().get(0).getReward());
                    BigDecimal fee = new BigDecimal(bsvMetadata.getData().get(0).getFee());
                    notice.setProfit(BigDecimal.valueOf(reward.doubleValue() + fee.doubleValue()));
                    homeNoticeService.saveOrUpdate(notice);
                    QueryWrapper<HomeNotice> queryWrapper =
                            new QueryWrapper<HomeNotice>();
                    queryWrapper
                            .orderByDesc("notice_timestamp")
                            .eq("coin_id", 98)
                            .last("limit 1");
                    List<HomeNotice> page = homeNoticeService.getBaseMapper().selectList(queryWrapper);
                    System.out.println("BSV getNoticeTimestamp:" + page.get(0).getNoticeTimestamp());
                    Coin coin = coinService.getById(98);
                    coin.setReward(reward);
                    coin.setLatestTime(page.get(0).getNoticeTimestamp());
                    boolean update = coinService.updateById(coin);
                    System.out.println("BSV update:" + update);
                }
            }
        });

    }


}
