package com.aopcloud.beam.app.scheduled;


import com.alibaba.fastjson.JSON;
import com.aopcloud.beam.app.api.app.entity.HomeNotice;
import com.aopcloud.beam.app.api.app.service.impl.HomeNoticeServiceImpl;
import com.aopcloud.beam.app.api.coin.entity.Coin;
import com.aopcloud.beam.app.api.coin.service.impl.CoinServiceImpl;
import com.aopcloud.beam.app.configurer.AppContextHolder;
import com.aopcloud.beam.app.scheduled.entity.BCHMetadata;
import com.aopcloud.beam.app.scheduled.entity.BSVMetadata;
import com.aopcloud.beam.app.scheduled.entity.BTCMetadata;
import com.aopcloud.beam.app.scheduled.entity.CoinCurrentPrice;
import com.aopcloud.beam.app.http.OkHttpUtils;
import com.aopcloud.beam.app.http.callback.StringCallback;
import okhttp3.Call;
import org.apache.commons.lang.StringUtils;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.List;

@Configuration      //1.主要用于标记配置类，兼备Component的效果。
@EnableScheduling   // 2.开启定时任务
public class CoinPriceScheduled {

    @Resource
    private CoinServiceImpl coinService;


//    @Scheduled(fixedDelay = 3 * 60 * 1000)
    private void configureTasks() {
        if (coinService == null) {
            coinService = AppContextHolder.getBean(CoinServiceImpl.class);
        }

        System.err.println("执行getCoinPrice  task  start : " + LocalDateTime.now());
        getCoinPrice();
        System.err.println("执行getCoinPrice task end : " + LocalDateTime.now());
    }

    private void getCoinPrice() {
        OkHttpUtils.get().url(coinPrice).build().execute(new StringCallback() {
            @Override
            public void onError(Call call, Exception e, int id) {

            }

            @Override
            public void onResponse(String response, int id) {
                if (StringUtils.isEmpty(response)) {
                    return;
                }
                List<CoinCurrentPrice> list = JSON.parseArray(response, CoinCurrentPrice.class);
                System.out.println(list.size()+"/"+1);

                for (int i = 0; i < list.size(); i++) {
                    CoinCurrentPrice currentPrice = list.get(i);
                    System.out.println(currentPrice.getSymbol()+"/"+currentPrice.getCurrentPrice());
                    BigDecimal price = new BigDecimal(currentPrice.getCurrentPrice());
                    if (currentPrice.getSymbol().equals("btc")) {
                        Coin coin = coinService.getById(96);
                        coin.setCurrentPrice(price);
                        coinService.updateById(coin);
                    } else if (currentPrice.getSymbol().equals("bch")) {
                        Coin coin = coinService.getById(97);
                        coin.setCurrentPrice(price);
                        coinService.updateById(coin);
                    } else if (currentPrice.getSymbol().equals("bsv")) {

                        Coin coin = coinService.getById(98);
                        coin.setCurrentPrice(price);
                        coinService.updateById(coin);
                    }
                }
            }
        });
    }


    String coinPrice = "https://api.coingecko.com/api/v3/coins/markets?vs_currency=cny&ids=bitcoin%2Cbitcoin-cash%2Cbitcoin-cash-sv&order=volume_desc&per_page=50&page=1&sparkline=false";


}
