package com.aopcloud.beam.app.util;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.OSSException;
import com.aliyun.oss.model.*;
import com.aopcloud.beam.app.common.OSSConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLDecoder;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;

public class AliyunOSSUtil {

    private static AliyunOSSUtil instance = null;


    public static AliyunOSSUtil getInstance() {
        if (null == instance) {
            instance = new AliyunOSSUtil();
        }
        return instance;
    }

    public AliyunOSSUtil() {
        init();
    }

    public static String upload(File file) {
        return upload(file, OSSConfig.OSS_FILE_HOST);
    }


    public static String uploadApk(File file) {
        return upload(file, OSSConfig.OSS_FILE_HOST);
    }

    public static String uploadImg(File file) {
        return upload(file, "img");
    }

    public static String upload(File file, String path) {
        Logger logger = LoggerFactory.getLogger(AliyunOSSUtil.class);
        String endpoint = OSSConfig.OSS_END_POINT;
        String accessKeyId = OSSConfig.OSS_ACCESS_KEY_ID;
        String accessKeySecret = OSSConfig.OSS_ACCESS_KEY_SECRET;
        String bucketName = OSSConfig.OSS_BUCKET_NAME;
        String fileHost = path;

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String dateStr = format.format(new Date());
        if (null == file) {
            return null;
        }
        OSSClient ossClient = new OSSClient(endpoint, accessKeyId, accessKeySecret);
        try {
            //容器不存在，就创建
            if (!ossClient.doesBucketExist(bucketName)) {
                ossClient.createBucket(bucketName);
                CreateBucketRequest createBucketRequest = new CreateBucketRequest(bucketName);
                createBucketRequest.setCannedACL(CannedAccessControlList.PublicRead);
                ossClient.createBucket(createBucketRequest);
            }

            //创建文件路径
            String fileUrl = fileHost + "/" + (dateStr + "/" + UUID.randomUUID().toString().replace("-", "") + "-");

            String fileName = URLDecoder.decode(file.getName().toString(), "UTF-8").replaceAll(" ", "");

            fileUrl = fileUrl + fileName;
            //上传文件
            PutObjectResult result = ossClient.putObject(new PutObjectRequest(bucketName, fileUrl, file));
            //设置权限 这里是公开读
            ossClient.setBucketAcl(bucketName, CannedAccessControlList.PublicRead);
            if (null != result) {

                return "https://" + bucketName + "." + endpoint + "/" + fileUrl;
            }
        } catch (OSSException | UnsupportedEncodingException oe) {
            logger.error(oe.toString());
        } finally {
            logger.error("ossClient  finally");
            //关闭
            ossClient.shutdown();
        }
        return null;
    }

    OSS ossClient;

    private void init() {
        ossClient = new OSSClientBuilder().build(OSSConfig.OSS_END_POINT, OSSConfig.OSS_ACCESS_KEY_ID, OSSConfig.OSS_ACCESS_KEY_SECRET);

    }


    public String getFileUrl(String objectName) {
        objectName = OSSConfig.OSS_IMG_PATH+objectName;
        boolean found = ossClient.doesObjectExist(OSSConfig.OSS_BUCKET_NAME, objectName);
        if (found) {
            return OSSConfig.OSS_DOMAIN + objectName;
        }
        return null;
    }

}
